/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.mdsp

import java.io.File

class DataGeneratorRunner(private val workingDir: File, private val outputDir: File) {
    fun generate() {
        if (workingDir.resolve("generated").exists()) {
            println("Output directory already exists, skipping.")
            return
        }

        val minecraftServerFile = workingDir.resolve("minecraft_server.jar")

        val javaBinDir = File(System.getProperty("java.home")).resolve("bin")
        val javaExecutable = javaBinDir.resolve("java")
            .run { if (exists()) this else javaBinDir.resolve("java.exe") }

        val process = ProcessBuilder()
            .command(
                javaExecutable.absolutePath,
                "-cp",
                minecraftServerFile.absolutePath,
                "net.minecraft.data.Main",
                "--server",
                "--reports"
            )
            .directory(workingDir)
            .start()

        process.waitFor()
    }
}
