/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.testplugin

import space.uranos.Position
import space.uranos.Uranos
import space.uranos.chat.TextComponent
import space.uranos.net.ServerListInfo
import space.uranos.net.event.ServerListInfoRequestEvent
import space.uranos.net.event.SessionAfterLoginEvent
import space.uranos.player.GameMode
import space.uranos.plugin.Plugin
import space.uranos.testplugin.anvil.AnvilWorld
import space.uranos.world.Dimension
import space.uranos.world.VoxelLocation
import space.uranos.world.block.CraftingTableBlock
import space.uranos.world.block.GreenWoolBlock
import space.uranos.world.block.RedWoolBlock

class TestPlugin: Plugin("Test", "1.0.0") {
    override fun onEnable() {
        val dimension = Dimension(
            "test:test",
            true,
            0.0f,
            true
        )

        Uranos.dimensionRegistry.register(dimension)

        val world = AnvilWorld(dimension, true)
        world.getVoxelsInCube(VoxelLocation.of(16, 0, 16), VoxelLocation.of(-16, 0, -16)).forEach {
            it.block = if (it.location.x % 16 == 0 || it.location.z % 16 == 0) GreenWoolBlock() else RedWoolBlock()
        }

        world.getVoxel(VoxelLocation.of(-1, 2, -1)).block = CraftingTableBlock()

        Uranos.eventBus.on<ServerListInfoRequestEvent> { event ->
            event.response = ServerListInfo("1.16.4", 754, TextComponent of "Test", 10, 0, emptyList())
        }

        Uranos.eventBus.on<SessionAfterLoginEvent> { event ->
            event.gameMode = GameMode.CREATIVE
            event.canFly = true
            event.flying = true
            event.initialWorldAndLocation = VoxelLocation
                .of(0, 2, 0)
                .atTopCenter()
                .withRotation(0f, 0f)
                .inside(world)
        }
    }
}
