/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.testplugin.anvil

import com.google.common.cache.LoadingCache
import space.uranos.player.Player
import space.uranos.util.createWeakValuesLoadingCache
import space.uranos.world.*
import space.uranos.world.block.AirBlock

class AnvilChunk(world: AnvilWorld, key: Key) : Chunk(world, key) {
    // TODO: Implement light

    override fun getData(player: Player?): ChunkData {
        return ChunkData(
            sections.map { section -> if (section.blocks.all { it == AirBlock }) null else section.blocks }
                .toTypedArray(),
            Array(ChunkData.BIOME_AREAS_IN_CHUNK) { Biome.PLAINS }
        )
    }

    override fun getLightData(player: Player?): ChunkLightData {
        return ChunkLightData(
            sections.map { it.skyLightValues }.toTypedArray(),
            sections.map { it.blockLightValues }.toTypedArray()
        )
    }

    val sections: Array<AnvilChunkSection> = Array(16) { AnvilChunkSection() }

    // Keys are relative to the world
    private val voxels: LoadingCache<VoxelLocation, Voxel> =
        createWeakValuesLoadingCache { location -> AnvilVoxel(location, world) }

    override fun getVoxelRelativeToChunk(location: VoxelLocation): Voxel {
        return voxels.get(key.translateChunkToWorld(location))
    }

    override fun getVoxel(location: VoxelLocation): Voxel {
        return voxels.get(location)
    }
}
