/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.testplugin.anvil

import space.uranos.world.Chunk
import space.uranos.world.Voxel
import space.uranos.world.VoxelLocation
import space.uranos.world.block.Block

class AnvilVoxel(
    override val location: VoxelLocation,
    override val world: AnvilWorld
) : Voxel() {
    private val chunkX: Int
    private val chunkY: UByte = location.y
    private val chunkZ: Int

    init {
        val (x, z) = chunk.key.translateWorldToChunk(location.x, location.z)
        chunkX = x
        chunkZ = z
    }

    private val sectionIndex = chunkY.toInt() / Chunk.SECTION_HEIGHT
    private val index = chunkY.toInt() * Chunk.AREA + chunkZ * Chunk.LENGTH + chunkX

    override var block: Block
        get() = (chunk as AnvilChunk).sections[sectionIndex].blocks[index]
        set(value) {
            (chunk as AnvilChunk).sections[sectionIndex].blocks[index] = value
        }
}
