/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.testplugin.anvil

import com.google.common.cache.CacheBuilder
import com.google.common.cache.CacheLoader
import com.google.common.cache.LoadingCache
import space.uranos.entity.Entity
import space.uranos.world.Chunk
import space.uranos.world.Dimension
import space.uranos.world.World
import java.util.*

class AnvilWorld(
    override val dimension: Dimension,
    override val isFlat: Boolean
) : World(UUID.randomUUID()) {
    override val loadedChunks: Map<Chunk.Key, Chunk> get() = chunks.asMap().filter { (_, chunk) -> chunk.loaded }
    override val entities = emptyList<Entity>()

    private val chunks: LoadingCache<Chunk.Key, Chunk> = CacheBuilder.newBuilder()
        .build(object : CacheLoader<Chunk.Key, Chunk>() {
            override fun load(key: Chunk.Key): Chunk {
                return AnvilChunk(this@AnvilWorld, key)
            }
        })

    override fun getChunk(key: Chunk.Key): Chunk = chunks.get(key)

    override fun spawnEntity(entity: Entity) {
        TODO("Not yet implemented")
    }
}
