/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos

import space.uranos.world.VoxelLocation
import space.uranos.world.World
import kotlin.math.roundToInt

/**
 * Represents a combination of x, y and z coordinates.
 */
data class Location(val x: Double, val y: Double, val z: Double) {
    /**
     * Converts this Location to a VoxelLocation by converting [x], [y] and [z] to integers using [Double.toInt],
     * in contrast to [roundToBlock] which uses [Double.roundToInt].
     */
    fun toVoxelLocation(): VoxelLocation = VoxelLocation(x.toInt(), y.toInt().toUByte(), z.toInt())

    /**
     * Converts this Location to a VoxelLocation by converting [x], [y] and [z] to integers using [Double.roundToInt],
     * in contrast to [toVoxelLocation] which uses [Double.toInt].
     */
    fun roundToBlock(): VoxelLocation = VoxelLocation(x.roundToInt(), y.roundToInt().toUByte(), z.roundToInt())

    fun withRotation(yaw: Float, pitch: Float) = Position(x, y, z, yaw, pitch)
    fun asVector() = Vector(x, y, z)

    infix fun inside(world: World): Pair<World, Location> = world to this

    operator fun get(part: CoordinatePart): Double = when (part) {
        CoordinatePart.X -> x
        CoordinatePart.Y -> y
        CoordinatePart.Z -> z
    }
}
