/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.event

import kotlin.contracts.InvocationKind
import kotlin.contracts.contract

interface Cancellable {
    var cancelled: Boolean
}

/**
 * Only executes [fn] if [isCancelled][Cancellable.cancelled] is true.
 */
inline fun <T : Cancellable, R> T.ifCancelled(fn: (T) -> R): R? {
    contract {
        callsInPlace(fn, InvocationKind.AT_MOST_ONCE)
//        returns(null) implies(isCancelled)
    }

    return if (cancelled) fn(this) else null
}

/**
 * Only executes [fn] if [isCancelled][Cancellable.cancelled] is false.
 */
inline fun <T : Cancellable, R> T.ifNotCancelled(fn: (T) -> R): R? {
    contract {
        callsInPlace(fn, InvocationKind.AT_MOST_ONCE)
//        returns(null) implies(isCancelled)
    }

    return if (!cancelled) fn(this) else null
}
