/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.event

import space.uranos.plugin.Plugin
import kotlin.reflect.KClass

data class EventHandler<T : Event> internal constructor(
    private val eventEmitter: EventEmitter<*>,
    val eventType: KClass<T>,
    val plugin: Plugin?,
    val handlerPosition: EventHandlerPosition,
    val fn: suspend (event: T) -> Unit
): Comparable<EventHandler<*>> {
    /**
     * Unregisters this event handler.
     */
    fun remove(): Boolean = eventEmitter.remove(this)

    override fun compareTo(other: EventHandler<*>): Int {
        val positionComparison = handlerPosition.compareTo(other.handlerPosition)

        return if (positionComparison == 0) hashCode() - other.hashCode() else positionComparison
    }
}
