/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet

import kotlin.reflect.KClass

abstract class Protocol constructor(val name: String, vararg codecs: PacketCodec<*>) {
    val codecs = codecs.toSet()
    private val codecsByPacketType = codecs.map { it.dataType to it }.toMap()
    val incomingPacketCodecsByID = codecs.filterIsInstance<IncomingPacketCodec<*>>().map { it.id to it }.toMap()

    @Suppress("UNCHECKED_CAST")
    fun <T : Packet> getCodecByType(type: KClass<T>): PacketCodec<T> = codecsByPacketType[type] as PacketCodec<T>?
        ?: throw Exception(
            "No packet codec found for this packet type ($type). " +
                    "This can happen if the packet is not part of the current protocol."
        )
}
