/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.util

import com.google.common.cache.CacheBuilder
import com.google.common.cache.CacheLoader
import com.google.common.cache.LoadingCache

/**
 * Creates a LoadingCache with weak values.
 *
 * @param loader The function invoked every time a value is not yet in the cache.
 */
fun <K, V> createWeakValuesLoadingCache(loader: (key: K) -> V): LoadingCache<K, V> {
    return CacheBuilder
        .newBuilder()
        .weakValues()
        .build(object : CacheLoader<K, V>() {
            override fun load(key: K): V = loader(key)
        })
}
