/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.util

import kotlin.reflect.KClass

class KClassToInstanceMap<K : Any> : MutableMap<KClass<out K>, K> by HashMap() {
    fun <T : K> getInstance(key: Key<T>): T? = getInstance(key.actualKey)

    @Suppress("UNCHECKED_CAST")
    fun <T : K> getInstance(key: KClass<T>): T? = get(key) as T

    fun <T : K> putInstance(key: Key<T>, value: T) = this.putInstance(key.actualKey, value)

    fun <T : K> putInstance(key: KClass<T>, value: T) = this.put(key, value)

    interface Key<T : Any> {
        val actualKey: KClass<T>
    }
}
