/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.util

import java.util.*

/**
 * Parses the UUID represented by this string and returns it.
 *
 * The difference to [UUID.fromString] is that this function also works on stringified UUIDs without hyphens.
 */
fun String.toUUID(): UUID {
    if (this.contains("-")) return UUID.fromString(this)

    val chars = this.toMutableList()
    chars.add(20, '-')
    chars.add(16, '-')
    chars.add(12, '-')
    chars.add(8, '-')

    return UUID.fromString(chars.joinToString(""))
}

/**
 * Returns the string representation of the UUID without hyphens.
 */
fun UUID.toStringWithoutHyphens() = toString().replace("-", "")
