/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.world

import space.uranos.Registry
import java.util.concurrent.atomic.AtomicInteger

class BiomeRegistry: Registry<Biome>() {
    private val nextID = AtomicInteger(0)

    override fun register(item: Biome): Boolean {
        item.numericID = nextID.getAndIncrement()
        return super.register(item)
    }

    init {
        register(Biome.PLAINS)
    }

    override fun unregister(id: String): Boolean {
        if (id == Biome.PLAINS.id) throw IllegalArgumentException("The plains biome cannot be removed")

        val biome = internalItems.remove(id)

        return if (biome == null) false
        else {
            biome.numericID = null
            true
        }
    }
}
