/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.world

import space.uranos.world.block.Block

/**
 * A voxel in a chunk of a world.
 *
 * There is always only one instance for every [location][VoxelLocation] in a world.
 */
abstract class Voxel {
    abstract val location: VoxelLocation
    abstract val world: World
    abstract var block: Block

    val chunk: Chunk get() = world.getChunk(location)

    /**
     * Shorthand for [chunk.loaded][Chunk.loaded].
     */
    val loaded: Boolean get() = chunk.loaded

    /**
     * Loads [chunk] if it is not yet loaded.
     */
    suspend fun loadAndWait() {
        chunk.startLoading()
        chunk.waitUntilLoaded()
    }
}
