/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.world.block

import kotlin.reflect.KClass

interface Material<T : Block> {
    val blockClass: KClass<T>
    val codec: BlockCodec<T>
    val id: String
    val hardness: Float
    val transparent: Boolean
    val filteredLight: Int
    val emittedLight: Int
    val collisionShape: CollisionShape

    companion object {
        /**
         * All materials, sorted by their numeric ID in ascending order.
         */
        val all = GENERATED_BLOCKS
        val byID = GENERATED_BLOCKS.map { it.id to it }.toMap()

        private val byClass = GENERATED_BLOCKS.map { it.blockClass to it }.toMap()

        @Suppress("UNCHECKED_CAST")
        fun <T: Block> byClass(blockClass: KClass<T>): Material<T>? = byClass[blockClass] as Material<T>?
        inline fun <reified T: Block> byClass(): Material<T>? = byClass(T::class)
    }
}
