/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet.play

import io.netty.buffer.ByteBuf
import space.uranos.net.MinecraftProtocolDataTypes.writeVarInt
import space.uranos.net.packet.OutgoingPacketCodec
import space.uranos.util.checkBit
import space.uranos.util.setBit

// TODO: Implement partial updates
object ChunkLightDataPacketCodec : OutgoingPacketCodec<ChunkLightDataPacket>(0x23, ChunkLightDataPacket::class) {
    private const val OUTSIDE_SECTIONS_MASK = 0b100000000000000001

    @OptIn(ExperimentalUnsignedTypes::class)
    override fun ChunkLightDataPacket.encode(dst: ByteBuf) {
        dst.writeVarInt(key.x)
        dst.writeVarInt(key.z)

        dst.writeBoolean(false) // Trust edges (?)

        val emptySkyLightMask = data.skyLightValues
            .foldIndexed(0) { i, acc, current -> acc.setBit(i, current?.sum() == 0.toUInt()) }

        val emptyBlockLightMask = data.blockLightValues
            .foldIndexed(0) { i, acc, current -> acc.setBit(i, current?.sum() == 0.toUInt()) }

        val skyLightMask = data.skyLightValues
            .foldIndexed(0) { i, acc, current -> acc.setBit(i, current != null && !emptySkyLightMask.checkBit(i)) }

        val blockLightMask = data.skyLightValues
            .foldIndexed(0) { i, acc, current -> acc.setBit(i, current != null && !emptyBlockLightMask.checkBit(i)) }

        dst.writeVarInt(skyLightMask shl 1)
        dst.writeVarInt(blockLightMask shl 1)
        dst.writeVarInt((emptySkyLightMask shl 1) or OUTSIDE_SECTIONS_MASK)
        dst.writeVarInt((emptyBlockLightMask shl 1) or OUTSIDE_SECTIONS_MASK)

        for (sectionValues in data.skyLightValues) {
            if (sectionValues === null) continue
            dst.writeVarInt(2048)
            sectionValues.forEach { dst.writeByte(it.toInt()) }
        }

        for (sectionValues in data.blockLightValues) {
            if (sectionValues === null) continue
            dst.writeVarInt(2048)
            sectionValues.forEach { dst.writeByte(it.toInt()) }
        }
    }
}
