/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet.play

import io.netty.buffer.ByteBuf
import space.uranos.Difficulty
import space.uranos.entity.LivingEntity
import space.uranos.net.MinecraftProtocolDataTypes.writeUUID
import space.uranos.net.MinecraftProtocolDataTypes.writeVarInt
import space.uranos.net.packet.OutgoingPacketCodec

object SpawnLivingEntityPacketCodec : OutgoingPacketCodec<SpawnLivingEntityPacket>(0x02, SpawnLivingEntityPacket::class) {
    @Suppress("DuplicatedCode")
    override fun SpawnLivingEntityPacket.encode(dst: ByteBuf) {
        dst.writeVarInt(entityID)
        dst.writeUUID(uuid)
        dst.writeVarInt(type.numericID)
        dst.writeDouble(position.x)
        dst.writeDouble(position.y)
        dst.writeDouble(position.z)
        dst.writeByte(position.yawIn256Steps.toInt())
        dst.writeByte(position.pitchIn256Steps.toInt())
        dst.writeByte(((headPitch / 360) * 256).toInt())
        dst.writeShort((velocity.x * 8000).toInt().toShort().toInt())
        dst.writeShort((velocity.y * 8000).toInt().toShort().toInt())
        dst.writeShort((velocity.z * 8000).toInt().toShort().toInt())
    }

    fun getPacketFromEntity(entity: LivingEntity) = SpawnLivingEntityPacket(
        entity.globallyUniqueNumericID,
        entity.uuid,
        entity.type,
        entity.position,
        entity.headPitch,
        entity.velocity
    )
}
