/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet.play

import io.netty.buffer.ByteBuf
import space.uranos.entity.Entity
import space.uranos.entity.ItemEntity
import space.uranos.entity.ObjectEntity
import space.uranos.net.MinecraftProtocolDataTypes.writeUUID
import space.uranos.net.MinecraftProtocolDataTypes.writeVarInt
import space.uranos.net.packet.OutgoingPacketCodec
import java.lang.IllegalArgumentException

object SpawnObjectEntityPacketCodec : OutgoingPacketCodec<SpawnObjectEntityPacket>(0x00, SpawnObjectEntityPacket::class) {
    @Suppress("DuplicatedCode")
    override fun SpawnObjectEntityPacket.encode(dst: ByteBuf) {
        dst.writeVarInt(entityID)
        dst.writeUUID(uuid)
        dst.writeVarInt(type.numericID)
        dst.writeDouble(position.x)
        dst.writeDouble(position.y)
        dst.writeDouble(position.z)
        dst.writeByte(position.yawIn256Steps.toInt())
        dst.writeByte(position.pitchIn256Steps.toInt())
        dst.writeInt(data)
        dst.writeShort((velocity.x * 8000).toInt().toShort().toInt())
        dst.writeShort((velocity.y * 8000).toInt().toShort().toInt())
        dst.writeShort((velocity.z * 8000).toInt().toShort().toInt())
    }

    fun getPacketFromEntity(entity: ObjectEntity) = SpawnObjectEntityPacket(
        entity.globallyUniqueNumericID,
        entity.uuid,
        entity.type,
        entity.position,
        getDataForEntity(entity),
        entity.velocity
    )

    fun getDataForEntity(entity: ObjectEntity): Int = when(entity) {
        is ItemEntity -> 1
        // TODO: Add remaining
        else -> throw IllegalArgumentException("Unknown entity type")
    }
}
