/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet.play

import io.netty.buffer.ByteBuf
import space.uranos.CardinalDirection
import space.uranos.Difficulty
import space.uranos.entity.PaintingEntity
import space.uranos.entity.PaintingMotive
import space.uranos.net.MinecraftProtocolDataTypes.writeUUID
import space.uranos.net.MinecraftProtocolDataTypes.writeVarInt
import space.uranos.net.MinecraftProtocolDataTypes.writeVoxelLocation
import space.uranos.net.packet.OutgoingPacketCodec
import space.uranos.world.Voxel
import space.uranos.world.VoxelLocation
import kotlin.math.max

object SpawnPaintingPacketCodec : OutgoingPacketCodec<SpawnPaintingPacket>(0x03, SpawnPaintingPacket::class) {
    override fun SpawnPaintingPacket.encode(dst: ByteBuf) {
        dst.writeVarInt(entityID)
        dst.writeUUID(uuid)
        dst.writeVarInt(motive.numericID)
        dst.writeVoxelLocation(centerLocation)
        dst.writeByte(when(direction) {
            CardinalDirection.NORTH -> 2
            CardinalDirection.SOUTH -> 0
            CardinalDirection.WEST -> 1
            CardinalDirection.EAST -> 3
        })
    }

    fun getPacketFromEntity(entity: PaintingEntity) = SpawnPaintingPacket(
        entity.globallyUniqueNumericID,
        entity.uuid,
        entity.motive,
        getCenterLocation(entity.topLeftLocation, entity.motive),
        entity.direction
    )

    private fun getCenterLocation(topLeftLocation: VoxelLocation, motive: PaintingMotive): VoxelLocation =
        topLeftLocation.copy(
            x = max(0, motive.width / 2) + topLeftLocation.x,
            z = motive.height / 2 + topLeftLocation.z
        )
}
