/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet.login

import space.uranos.net.packet.IncomingPacket

/**
 * Sent by the client in response to a [LoginPluginRequestPacket].
 *
 * **See [LoginPluginRequestPacket] for more info.**
 */
data class LoginPluginResponsePacket(
    val messageID: Int,
    val data: ByteArray?
) : IncomingPacket() {
    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (javaClass != other?.javaClass) return false

        other as LoginPluginResponsePacket

        if (messageID != other.messageID) return false
        if (data != null) {
            if (other.data == null) return false
            if (!data.contentEquals(other.data)) return false
        } else if (other.data != null) return false

        return true
    }

    override fun hashCode(): Int {
        var result = messageID
        result = 31 * result + (data?.contentHashCode() ?: 0)
        return result
    }
}
