/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet.status

import space.uranos.chat.LegacyFormattingCode
import space.uranos.chat.TextComponent
import space.uranos.net.ServerListInfo
import space.uranos.net.packet.OutgoingPacket

/**
 * Sent by the server in response to a [RequestPacket][space.uranos.net.packet.status].
 *
 * @param versionName The name of the Minecraft version the server uses.
 * @param protocolVersion The number of the [protocol version](https://wiki.vg/Protocol_version_numbers) used by the server.
 * @param description The description of the server. Although this is a [TextComponent], [LegacyFormattingCode]s must be used.
 * @param players The players shown when hovering over the player count.
 * @param favicon The favicon of the server. This must be a base64 encoded 64x64 PNG image.
 */
data class ResponsePacket(
    val response: ServerListInfo
) : OutgoingPacket()
