/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.event

import space.uranos.logging.Logger
import space.uranos.util.pluralizeWithCount
import kotlin.system.measureTimeMillis

class UranosEventBus(private val developmentMode: Boolean) : EventBus() {
    private val logger = Logger("EventBus")

    /**
     * Invokes all previously registered event handlers sorted by their handlerPosition.
     *
     * The coroutine context is inherited.
     *
     * @return [event]
     */
    override suspend fun <T : Event> emit(event: T): T {
        if (developmentMode) {
            var count = 0
            val time = measureTimeMillis {
                for (handler in handlers) {
                    if (handler.eventType.isInstance(event)) {
                        @Suppress("UNCHECKED_CAST")
                        handler as EventHandler<T>

                        count++
                        handler.fn(event)
                    }
                }
            }

            logger trace "Emitted ${event::class.java.simpleName} to " +
                    "${pluralizeWithCount("handler", count)}, took ${time}ms"
        } else {
            for (handler in handlers) {
                if (handler.eventType.isInstance(event)) {
                    @Suppress("UNCHECKED_CAST")
                    handler as EventHandler<T>

                    handler.fn(event)
                }
            }
        }

        return event
    }
}
