/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.event

class UranosEventHandlerPositionManager: EventHandlerPositionManager {
    internal val positions = ArrayList<EventHandlerPosition>()

    init {
        positions.add(EventHandlerPosition.FIRST)
        positions.add(EventHandlerPosition.NORMAL)
        positions.add(EventHandlerPosition.LAST)
    }

    override fun positionOfOrNull(eventHandlerPosition: EventHandlerPosition): Int? =
        positions.indexOf(eventHandlerPosition).let { if (it == -1) null else it }

    override fun positionOf(eventHandlerPosition: EventHandlerPosition): Int = positionOfOrNull(eventHandlerPosition)
        ?: throw IllegalArgumentException("eventHandlerPosition was not registered")

    override fun insert(vararg positions: EventHandlerPosition) {
        insertBefore(EventHandlerPosition.LAST, *positions)
    }

    override fun insertBefore(existing: EventHandlerPosition, vararg positions: EventHandlerPosition) {
        if (existing == EventHandlerPosition.FIRST)
            throw IllegalArgumentException("You cannot insert positions before EventHandlerPosition.FIRST")

        val index = positionOfOrNull(existing) ?: throw IllegalArgumentException("existing was not registered")
        insertWithCheck(index, positions.distinct())
    }

    override fun insertAfter(existing: EventHandlerPosition, vararg positions: EventHandlerPosition) {
        if (existing == EventHandlerPosition.LAST)
            throw IllegalArgumentException("You cannot insert positions after EventHandlerPosition.LAST")

        val index = positionOfOrNull(existing) ?: throw IllegalArgumentException("existing was not registered")
        insertWithCheck(index + 1, positions.distinct())
    }

    private fun insertWithCheck(index: Int, positions: List<EventHandlerPosition>) {
        for (position in positions) {
            if (this.positions.indexOf(position) != -1) throw IllegalArgumentException("$position was already registered")
        }

        this.positions.addAll(index, positions)
    }
}
