/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net

import io.netty.buffer.ByteBuf
import io.netty.buffer.Unpooled
import java.nio.ByteBuffer
import javax.crypto.Cipher
import javax.crypto.SecretKey
import javax.crypto.spec.IvParameterSpec

class CryptBuffer(key: SecretKey, decrypt: Boolean) {
    private val cipher: Cipher = Cipher.getInstance("AES/CFB8/NoPadding")

    init {
        cipher.init(if (decrypt) Cipher.DECRYPT_MODE else Cipher.ENCRYPT_MODE, key, IvParameterSpec(key.encoded))
    }

    fun transform(buf: ByteBuf, out: MutableList<Any>) {
        val outBuffer = ByteBuffer.allocate(buf.readableBytes())
        cipher.update(buf.nioBuffer(), outBuffer)

        outBuffer.flip()
        out.add(Unpooled.wrappedBuffer(outBuffer))
    }
}
