/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net

import io.netty.buffer.ByteBuf
import io.netty.channel.ChannelHandlerContext
import io.netty.handler.codec.MessageToMessageCodec
import javax.crypto.SecretKey

class EncryptionCodec(val session: UranosSession, key: SecretKey) : MessageToMessageCodec<ByteBuf, ByteBuf>() {
    private val encryptionBuffer = CryptBuffer(key, false)
    private val decryptionBuffer = CryptBuffer(key, true)

    override fun encode(ctx: ChannelHandlerContext, msg: ByteBuf, out: MutableList<Any>) {
        encryptionBuffer.transform(msg, out)
    }

    override fun decode(ctx: ChannelHandlerContext, msg: ByteBuf, out: MutableList<Any>) {
        decryptionBuffer.transform(msg, out)
    }
}
