/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net

import io.netty.channel.Channel
import io.netty.channel.ChannelException
import io.netty.channel.ChannelInitializer
import io.netty.channel.ChannelOption
import space.uranos.UranosServer

class UranosChannelInitializer(private val server: UranosServer) : ChannelInitializer<Channel>() {
    override fun initChannel(channel: Channel) {
        if (!ipTOSFailed) try {
            channel.config().setOption(ChannelOption.IP_TOS, 0x18)
        } catch (e: ChannelException) {
            ipTOSFailed = true
            server.logger warn "Your OS does not support IP type of service"
        }

        channel.pipeline().addLast("framing", FramingCodec(server))
    }

    companion object {
        private var ipTOSFailed = false
    }
}
