/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet.play

import space.uranos.net.PacketReceivedEventHandler
import space.uranos.net.UranosSession
import space.uranos.player.UranosPlayer
import space.uranos.player.event.PlayerSettingsUpdateEvent

object ClientSettingsPacketHandler : PacketReceivedEventHandler<ClientSettingsPacket>() {
    override suspend fun handle(session: UranosSession, packet: ClientSettingsPacket) {
        val player = session.player

        if (player == null) {
            try {
                session.joinProcedure.onClientSettingsReceived(packet)
            } catch (e: IllegalStateException) {
                session.failAndDisconnectBecauseOfClient("Client sent ClientSettingsPacket when it was not allowed")
            }
        } else {
            (player as UranosPlayer).settings =
                session.server.eventBus.emit(PlayerSettingsUpdateEvent(player, packet.asPlayerSettings())).settings
        }
    }
}
