/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet.play

import io.netty.buffer.ByteBuf
import io.netty.buffer.Unpooled
import space.uranos.event.ifNotCancelled
import space.uranos.net.MinecraftProtocolDataTypes.readString
import space.uranos.net.PacketReceivedEventHandler
import space.uranos.net.UranosSession
import space.uranos.net.event.ClientBrandReceivedEvent

object IncomingPluginMessagePacketHandler : PacketReceivedEventHandler<IncomingPluginMessagePacket>() {
    override suspend fun handle(session: UranosSession, packet: IncomingPluginMessagePacket) {
        // TODO: Add API for registering plugin channels and using them

        if (packet.channel == "minecraft:brand") {
            if (session.brand != null) session.failAndDisconnectBecauseOfClient("The client sent his brand name more than once")
            else {
                val buffer: ByteBuf = Unpooled.copiedBuffer(packet.data)
                val brand = buffer.readString()
                buffer.release()

                session.server.eventBus.emit(ClientBrandReceivedEvent(session, brand)).ifNotCancelled { session.brand = it.brand }
            }
        }
    }
}
