/*
 * Copyright 2020-2021 Moritz Ruth and Uranos contributors
 * Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file
 */

package space.uranos.net.packet.status

import space.uranos.net.PacketReceivedEventHandler
import space.uranos.net.ProtocolPacketReceivedEventHandler
import space.uranos.net.event.ServerListInfoRequestEvent

object StatusProtocolHandler : ProtocolPacketReceivedEventHandler(mapOf(
    RequestPacket::class to PacketReceivedEventHandler.of<RequestPacket> { session, _ ->
        val event = session.server.eventBus.emit(ServerListInfoRequestEvent(session))
        val response = event.response

        when {
            event.cancelled -> session.disconnect(internalReason = "The ServerListInfoRequestEvent was cancelled.")
            response == null ->
                session.disconnect(internalReason = "The response property of the ServerListInfoRequestEvent is null.")

            else -> session.send(ResponsePacket(response))
        }
    },
    PingPacket::class to PacketReceivedEventHandler.of<PingPacket> { session, packet ->
        session.send(PongPacket(packet.payload))
        session.disconnect()
    }
))
